﻿import util.Observable;
import logger.LogMessage;

/** 
* 一個通用的日誌類別。採用getLog( )來建立能用在整個應用程式的實體。
* 藉由fatal( )、error( )、warn( )、info( )和debug( )傳送訊息。
* 透過addObserver( )新增日誌的view（所有的view都必須實作Observer）。
*  
* @version 1.0.0 
*/ 
class logger.Logger extends Observable {
  // 靜態變數。參考到log實體（單態）。
  private static var log:Logger = null;

  // 訊息的日誌等級數值
  public static var FATAL:Number = 0;
  public static var ERROR:Number = 1;
  public static var WARN:Number  = 2;
  public static var INFO:Number  = 3;
  public static var DEBUG:Number = 4;

  private var lastMsg:LogMessage;

  // 方便人們閱讀的日誌等級說明文字。
  public static var levelDescriptions = ["FATAL", "ERROR", 
                                         "WARN", "INFO", "DEBUG"];

  // log的等級篩檢設定（等級數字從0開始）。帶有大於logLevel
  // 等級值的訊息，將不會被傳遞給觀察者
  // 預設值為3，"INFO"（只有DEBUG訊息被濾除）。
  private var logLevel:Number;

  /**
   * Logger建構子
  */
  private function Logger (  ) {
    // 預設只顯示"INFO"等級的訊息
    setLevel(Logger.INFO);   
  }

  /**
   * 傳回log實體的參考。
   * 如果沒有log實體存在，則建立一個。
   *
   * @return   傳回一個Logger實體。
   */
  public static function getLog():Logger {
    if (log == null) {
      log = new Logger();
    }
    return log;  
  }

  /**
   * 傳回方便人們閱讀，代表指定日誌等級的字串。
   */
  public static function getLevelDesc(level:Number):String {
    return levelDescriptions[level];
  }

  /**
   * 設定log的訊息篩檢等級。
   *
   * @param   lev   超過這個參數以上的等級的訊息將被濾除。
   */
  public function setLevel(lev:Number):Void {
    // 確保傳入的等級參數是個整數
    lev = Math.floor(lev);
    // 如果此參數值是被認可的等級值，就設定日誌的等級。
    if (lev >= Logger.FATAL && lev <= Logger.DEBUG) {
      logLevel = lev;
      info("日誌的等級被設定成：" + lev);
      return;
    }
    // 如果程式執行到這裡，就代表日誌等級值不合法。 
    warn("指定了不合法的日誌等級。");
  }

  /**
   * 傳回日誌的訊息篩檢等級設定。
   */
  public function getLevel():Number {
    return logLevel;
  }

  /**
   * 傳回傳送給日誌的最新訊息
   */
  public function getLastMsg():LogMessage {
    return lastMsg;
  }

  /**
   * 傳給日誌一則嚴重等級為"FATAL（致命）"的訊息。
   */
  public function fatal(msg:String):Void {
    // 如果篩檢等級至少是"FATAL"，
    // 就廣播訊息給觀察者。
    if (logLevel >= Logger.FATAL) {
      // 建構日誌訊息物件
      lastMsg = new LogMessage(msg, Logger.FATAL);

      // 把訊息傳給觀察者
      setChanged(  );
      notifyObservers(lastMsg);
    }
  }

  /**
   * 傳給日誌一則嚴重等級為"ERROR（嚴重）"的訊息。 
   */
  public function error(msg:String):Void {
    // 如果篩檢等級至少是"ERROR"， 
    // 就廣播訊息給觀察者。
    if (logLevel >= Logger.ERROR) {
      lastMsg = new LogMessage(msg, Logger.ERROR);

      setChanged(  );
      notifyObservers(lastMsg);
    }
  }

  /**
   * 傳給日誌一則嚴重等級為"WARN（警告）"的訊息。 
   */
  public function warn(msg:String):Void {
    // 如果篩檢等級至少是"WARN"， 
    // 就廣播訊息給觀察者。
    if (logLevel >= Logger.WARN) {
      lastMsg = new LogMessage(msg, Logger.WARN);

      setChanged(  );
      notifyObservers(lastMsg);
    }
  }
  
  /**
   * 傳給日誌一則嚴重等級為"INFO（資訊）"的訊息。 
   */
  public function info(msg:String):Void {
    // 如果篩檢等級至少是"INFO"， 
    // 就廣播訊息給觀察者。
    if (logLevel >= Logger.INFO) {
      lastMsg = new LogMessage(msg, Logger.INFO);

      setChanged(  );
      notifyObservers(lastMsg);
    }
  }

  /**
   * 傳給日誌一則嚴重等級為"DEBUG（除錯）"的訊息。 
   */
  public function debug(msg:String):Void {
    // 如果篩檢等級至少是"DEBUG"， 
    // 就廣播訊息給觀察者。
    if (logLevel >= Logger.DEBUG) {
      lastMsg = new LogMessage(msg, Logger.DEBUG);

      setChanged(  );
      notifyObservers(lastMsg);
    }
  }
}